import 'package:maybe/maybe.dart';
import "package:test/test.dart";

void main() {
  group("Maybe", () {
    group(".when", () {
      test("[some] is called with some value", () {
        // Defining a value
        var maybe = Maybe.some("hello world");
        var actual = "";
        Maybe.when(maybe,
            nothing: () => actual = "nothing",
            some: (String v) {
              actual = v;
            });
        expect(actual, equals("hello world"));
      });

      test("[nothing] isn't called with some value", () {
        // Defining a value
        var maybe = Maybe.some("hello world");
        var actual = "";
        Maybe.when(maybe,
            nothing: () => actual = "nothing", some: (v) => "some");
        expect(actual, equals(""));
      });

      test("[some] isn't called with nothing", () {
        // Defining a value
        var maybe = Maybe<String>.nothing();
        var actual = "";
        Maybe.when(maybe, some: (v) {
          actual = "some";
        });
        expect(actual, equals(""));
      });

      test("[nothing] is called with nothing", () {
        // Defining a value
        var maybe = Maybe<String>.nothing();
        var actual = "";
        Maybe.when(maybe, nothing: () {
          actual = "nothing";
        });
        expect(actual, equals("nothing"));
      });

      test("[nothing] is called with null", () {
        // Defining a value
        var actual = "";
        Maybe.when(null, nothing: () {
          actual = "nothing";
        });
        expect(actual, equals("nothing"));
      });

      test("[some] isn't called with null", () {
        // Defining a value
        var actual = "";
        Maybe.when(null, some: (v) {
          actual = "some";
        });
        expect(actual, equals(""));
      });

      test("[some] is called with nothing and defaultValue", () {
        // Defining a value
        var maybe = Maybe<String>.nothing();
        var actual = "";
        Maybe.when(maybe,
            nothing: () {
              actual = "nothing";
            },
            some: (v) {
              actual = v;
            },
            defaultValue: () => "default");
        expect(actual, equals("default"));
      });

      test("[nothing] is called when nothingWhen is verified", () {
        // Defining a value
        var maybe = Maybe.some("nothing", nothingWhen: (v) => v == "nothing");
        var actual = "";
        Maybe.when(maybe, some: (v) {
          actual = "some";
        });
        expect(actual, equals(""));
      });
    });

    group(".value", () {
      test("(nothing) returns defaultValue", () {
        // Defining a value
        var maybe = Maybe<String>.nothing();
        expect(Maybe.value(maybe, "default"), equals("default"));
      });
      
      test("(null) returns defaultValue", () {
        // Defining a value
        expect(Maybe.value(null, "default"), equals("default"));
      });

      test("(some) returns its value", () {
        // Defining a value
        var maybe = Maybe.some("value");
        expect(Maybe.value(maybe, "default"), equals("value"));
      });
    });

    group(".flatten", () {
      test("(nothing) creates merged 'nothing'", () {
        // Defining a value
        var maybe = Maybe<Maybe<String>>.nothing();
        var flatten = Maybe.flatten(maybe);
        var actual = "";
        Maybe.when<String>(flatten, nothing: () => actual = "nothing");
        expect(actual, equals("nothing"));
      });

      test("(some/nothing) creates merged 'nothing'", () {
        // Defining a value
        var maybe = Maybe.some(Maybe<String>.nothing());
        var flatten = Maybe.flatten(maybe);
        var actual = "";
        Maybe.when<String>(flatten, nothing: () => actual = "nothing");
        expect(actual, equals("nothing"));
      });

      test("(some/some) creates merged 'some'", () {
        // Defining a value
        var maybe = Maybe.some(Maybe<String>.some("some"));
        var flatten = Maybe.flatten(maybe);
        var actual = "";
        Maybe.when<String>(flatten, some: (v) => actual = v);
        expect(actual, equals("some"));
      });
    });
  });
}
